@@ -25,19 +25,15 @@ def stylus(cmd='stylus', out=None, fns=None, watch=False, root_name=None):
 
     logger.debug("Running stylus function.")
 
-    if out:
-        import os
-        if not os.path.exists(out):
-            os.makedirs(out)
-
     files = [f for f in get_files('**.styl')]
 
     if files:
-        build_run(cmd + ' ' + ' '.join(files))
+        for f in files:
+            _run_stylus_command(cmd, out, root_name, f)
 
     if watch:
+
         def callback(event, watcher):
-            import os
             path = None
             if event.src_path.endswith('.styl'):
                 path = event.src_path
@@ -45,13 +41,32 @@ def stylus(cmd='stylus', out=None, fns=None, watch=False, root_name=None):
                 path = event.dest_path
 
             if path:
-                if out and root_name:
-                    folders = path.partition(root_name)[2]
-                    run_cmd = cmd + " --out " + os.path.join(out, folders)
-                elif out:
-                    run_cmd = cmd + " --out " + out
+                _run_stylus_command(cmd, out, root_name, path)
 
-                build_run(run_cmd + ' ' + path)
+        def predicate(event, watcher):
+            if event.src_path.endswith('.styl'):
+                return True
+            elif getattr(event, 'dest_path', '').endswith('.styl'):
+                return True
+            return False
 
-        fab_watch(fns, '.', callback=callback, recursive=True, throttle=0)
+        fab_watch(fns=fns, callback=callback, throttle=1, predicate=predicate)
 
+
+def _run_stylus_command(cmd, out, root_name, path):
+    import os
+    if out:
+        if root_name and root_name + '/' in path:
+            # /root_name/1/2/test.styl
+            # 1/2/test.styl
+            # 1/2
+            folders = path.partition(root_name + '/')[2].rpartition('/')[0]
+            if folders:
+                out = os.path.join(out, folders)
+
+        if not os.path.exists(out):
+            os.makedirs(out)
+
+        cmd = cmd + " --out " + out
+
+    build_run(cmd + ' ' + path)
